namespace Syntriax.Engine.Core.Abstract;

/// <summary>
/// Represents an entity with an enable state that can be toggled.
/// </summary>
public interface IStateEnable : IHasEntity
{
    /// <summary>
    /// Event triggered when the <see cref="Enabled"/> state of the <see cref="IStateEnable"/> changes.
    /// </summary>
    event NameChangedEventHandler? OnEnabledChanged;

    /// <summary>
    /// The value indicating whether the <see cref="IStateEnable"/> is enabled.
    /// </summary>
    bool Enabled { get; set; }

    delegate void NameChangedEventHandler(IStateEnable sender, bool previousState);
}
